/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.client;

import java.util.HashMap;
import java.util.Map;

import javax.security.auth.login.FailedLoginException;

import org.ow2.petals.tools.jmx.api.PetalsJMXClient;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientInitializationException;
import org.ow2.petals.tools.webadmin.datacollector.client.service.AdminService;
import org.ow2.petals.tools.webadmin.datacollector.client.service.DeployService;
import org.ow2.petals.tools.webadmin.datacollector.client.service.EndpointService;
import org.ow2.petals.tools.webadmin.datacollector.client.service.InstallService;
import org.ow2.petals.tools.webadmin.datacollector.client.service.MonitoringService;
import org.ow2.petals.tools.webadmin.datacollector.client.service.SystemInfoService;
import org.ow2.petals.tools.webadmin.datacollector.client.service.TransportMonitoringService;
import org.ow2.petals.tools.webadmin.datacollector.util.StringHelper;

/**
 * @author mcarpentier - EBM WebSourcing
 * 
 * A {@link CollectorClient} allows you to connect to a petals server through
 * JMX. A {@link CollectorClient} should be built for each petals server to
 * admin
 * 
 */
public class CollectorClient {

    private final static int CONNECTION_FAIL_LIMIT = 3;

    /** common JMX client for all collector services * */
    private PetalsJMXClientBean petalsJMXClientBean;

    private PetalsJMXClient petalsJMXClient;

    /** collector services * */
    private MonitoringService monitoringService;

    private AdminService adminService;

    private DeployService deployService;

    private InstallService installService;

    private SystemInfoService systemInfoService;

    private TransportMonitoringService transportMonitoringService;

    private EndpointService endpointService;

    /** attributes of associated petals server * */
    private Map<String, String> serverProperties;

    /** for connection error check* */
    private int connectionFailedCount = 0;

    /**
     * @param petalsJMXServer
     * @param petalsJMXPort
     * @param petalsJMXPassword
     * @param petalsJMXLogin
     * @throws DataCollectorClientInitializationException
     * @throws DataCollectorClientException
     * @throws ConnectionErrorException
     * @throws NumberFormatException
     */
    public CollectorClient(final String petalsJMXServer, final String petalsJMXPort,
            final String petalsJMXLogin, final String petalsJMXPassword)
            throws DataCollectorClientInitializationException, DataCollectorClientException {

        try {
            if (StringHelper.isNullOrEmpty(petalsJMXServer)) {
                throw new DataCollectorClientInitializationException(
                        "You must specify an host to connect to the petals jmx server");
            }
            if (StringHelper.isNullOrEmpty(petalsJMXPort)) {
                throw new DataCollectorClientInitializationException(
                        "You must specify a port to connect to the petals jmx server");
            }
            this.petalsJMXClientBean = new PetalsJMXClientBean(petalsJMXServer, petalsJMXPort,
                    petalsJMXPassword, petalsJMXLogin);
            this.serverProperties = new HashMap<String, String>();

            this.petalsJMXClient = new PetalsJMXClient(this.petalsJMXClientBean
                        .getPetalsJMXServer(), this.petalsJMXClientBean.getPetalsJMXport(),
                        this.petalsJMXClientBean.getPetalsJMXLogin(), this.petalsJMXClientBean
                                .getPetalsJMXPassword());

            this.petalsJMXClient = new PetalsJMXClient(petalsJMXServer, Integer
                    .parseInt(petalsJMXPort), petalsJMXLogin, petalsJMXPassword);
            
        } catch (final ConnectionErrorException e) {
            String errMsg = "Can't connect to PEtALS server ("
                    + this.petalsJMXClientBean.getPetalsJMXServer() + ":"
                    + this.petalsJMXClientBean.getPetalsJMXport() + ")";
            if ((e.getCause() != null) && (e.getCause().getCause() != null)
                    && (e.getCause().getCause() instanceof FailedLoginException)) {
                errMsg = errMsg + " : Invalid username or password";
            }
            throw new DataCollectorClientException(errMsg, e);

        }
    }

    public AdminService getAdminService() {
        if (this.adminService == null) {
            this.adminService = new AdminService(this.petalsJMXClient);
        }
        return this.adminService;
    }

    public DeployService getDeployService() {
        if (this.deployService == null) {
            this.deployService = new DeployService(this.petalsJMXClient);
        }
        return this.deployService;
    }

    public EndpointService getEndpointService() {
        if (this.endpointService == null) {
            this.endpointService = new EndpointService(this.petalsJMXClient);
        }
        return this.endpointService;
    }

    public InstallService getInstallService() {
        if (this.installService == null) {
            this.installService = new InstallService(this.petalsJMXClient);
        }
        return this.installService;
    }

    public MonitoringService getMonitoringService() {
        if (this.monitoringService == null) {
            this.monitoringService = new MonitoringService(this.petalsJMXClient);
        }
        return this.monitoringService;
    }

    public SystemInfoService getSystemInfoService() {
        if (this.systemInfoService == null) {
            this.systemInfoService = new SystemInfoService(this.petalsJMXClient);
        }
        return this.systemInfoService;
    }

    public TransportMonitoringService getTransportMonitoringService() {
        if (this.transportMonitoringService == null) {
            this.transportMonitoringService = new TransportMonitoringService(this.petalsJMXClient);
        }
        return this.transportMonitoringService;
    }

    public Map<String, String> getServerProperties() {
        return serverProperties;
    }

    public void addServerProperty(final String property, final String value) {
        if (this.serverProperties == null) {
            this.serverProperties = new HashMap<String, String>();
        }
        this.serverProperties.put(property, value);
    }

    public String getServerProperty(String property) {
        if (this.serverProperties == null) {
            this.serverProperties = new HashMap<String, String>();
        }
        return this.serverProperties.get(property);
    }

    /**
     * This method check if ther's not too many connection failed AND inc
     * connection fail count
     * 
     * @return false if connection count has reach the limit
     */
    public boolean checkConnectionFailedCount() {
        this.connectionFailedCount++;
        return (this.connectionFailedCount < CONNECTION_FAIL_LIMIT);
    }

    public void resetConnectionFailedCount() {
        this.connectionFailedCount = 0;
    }

    public int getConnectionFailedCount() {
        return connectionFailedCount;
    }

}
